-- Import necessary modules for Android speech recognition and text manipulation
require "import"
import "android.speech.RecognizerIntent"
import "android.speech.SpeechRecognizer"
import "android.speech.RecognitionListener"
import "com.androlua.*"
import "android.net.Uri"
import "android.content.Intent"

-- Define a variable to store the spoken words
local spokenText = ""

-- Function to store recognized words
function saveWords(words)
    spokenText = words
end

-- Function to start listening for speech input
function startListening()
    local verbalizationDetected = false

    -- Create a new intent for speech recognition
    local recognizerIntent = Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL, RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_LANGUAGE, "bn_IN") -- Set the source language to Bengali (bn_IN)
    recognizerIntent.putExtra(RecognizerIntent.EXTRA_CALLING_PACKAGE, this.getPackageName())

    -- Create a speech recognizer instance
    local speechRecognizer = SpeechRecognizer.createSpeechRecognizer(this)

    -- Define a recognition listener for speech events
    local speechListener = RecognitionListener{
        onResults = function(results)
            local data = results.getParcelableArrayList(SpeechRecognizer.RESULTS_RECOGNITION)
            if data ~= nil and data.size() > 0 then
                verbalizationDetected = true
                local recognizedText = data.get(0)

                -- Save the recognized words
                saveWords(recognizedText)

                -- Get the EditText node
                local node = service.getEditText()
                local editText = service.getText(node)

                -- Define the target language as English
                local targetLanguage = "english"
                local defaultLanguage = Locale.getDefault().getLanguage()
                local languageCodes = { ["english"] = "en" } -- Define language code for English
                
                -- Retrieve the language code for English
                local languageCode = languageCodes[targetLanguage]
                if languageCode then
                    local wordsToTranslate = recognizedText
                    local translatedWords = ""
                    
                    -- Construct the URL for translation
                    local url = "https://translate.google.com/m?sl=bn&hl=" .. defaultLanguage .. "&q=" .. Uri.encode(wordsToTranslate)
                    url = url .. "&tl=" .. languageCode
                    
                    -- Make a GET request to Google Translate for translation
                    Http.get(url, function(status, result)
                        if status == 200 then
                            -- Extract the translated words from the response
                            translatedWords = result:match('<div class="result%-container">([^<]+)</div>')
                            -- Insert translated words into the EditText node
                            service.insertText(node, translatedWords .. " ")
                            -- Speak the translated words
                            this.speak(translatedWords)
                        end
                    end)
                end
            end

            -- If no verbalization detected, stop listening
            if not verbalizationDetected then
                kill()
                return false
            end
        end,
        onError = function()
            -- If an error occurs, stop listening
            kill()
            return false
        end
    }

    -- Set the recognition listener for the speech recognizer
    speechRecognizer.setRecognitionListener(speechListener)
    -- Start listening for speech input
    speechRecognizer.startListening(recognizerIntent)
end

-- Call the function to start listening for speech input
startListening()
-- Return true to indicate successful execution
return true
